from collections import defaultdict

from djangotools.cmdline import Command, CronManager, Argument, CommandData

from djangotools.cmdline import tasks
from djangotools.config import get_settings
from djangotools.run import Migrate


class InstallCommand(Command):
    NAME = "install"
    ALIASES = ["i"]

    HELP = "Lance le script d'installation"
    ARGUMENT = [
        Argument("--no-interative", "-I", action="store_true", help="Ne demande pas d'inforamtion à l'utilisateur"),
        Argument("--user", "-u", help="Le nom d'utilisateur s'il n'existe aucun utilisateur"),
        Argument("--mail", "-m", help="Le mail s'il n'existe aucun utilisateur"),
        Argument("--password", "-p", help="Le mot de passe s'il n'existe aucun utilisateur"),
    ]


    def migrate(self, data):
        handler = Migrate()
        handler.handle()

    def _prompt(self, prompt, required):
        while True:
            ret = input(f"{prompt}: ")
            ret = ret.split("\n")[0] if isinstance(ret, str) else ret
            if not ret:
                if required: continue
                else: return ret
            return ret

    def prompt(self, data, name, prompt, required):
        return getattr(data, name) or self._prompt(prompt, required)

    def create_user(self, data):
        from django.contrib.auth.models import User
        if len(User.objects.all()) > 0: return
        username = self.prompt(data, "user", "Nom d'utilisateur", True)
        password = self.prompt(data, "password", "Mot de passe", True)
        email = self.prompt(data, "mail", "Mail", False)
        print(f"Création du super utilisateur {username}")

        User.objects.create_user(username=username, password=password, email=email, is_superuser=True, is_staff=True)


    def run(self, data):
        self.migrate(data)
        self.create_user(data)


