import json
import traceback

from django.conf import settings
from django.contrib.auth import login, authenticate, logout
from django.contrib.auth.models import User
from django.http import HttpResponseRedirect
from django.shortcuts import render

from djangotools.common import response, get_url
from djangotools.common.errors import CustomException, UnauthorizedException
from djangotools.views.context.base import get_context_class
from djangotools.views.router import Router
from djangotools.common.loggin import debug_login


def context(page, data, **kwargs):
    return {
        "context": json.dumps({
            "page" : page,
            "data" : data,
            **kwargs
        })
    }



@Router.post(settings.CONTEXT_URL, )
def raw_context(request, name):
    debug_login(request)
    context_class = get_context_class(name)

    data = json.loads(request.body) or {}
    url = data.get("url_data", {})
    post = data.get("post", {})
    if context_class is None:
        raise ValueError(f"Impossible de trouver le contexte : {name}")
    context = context_class()

    try:
        data = context.get_context(request, **url, post_action=post)
    except UnauthorizedException as err:
        return response.serv_json_not_logged()


    return response.serv_json_ok(data)


def _render(request, name, page=settings.ROOT_TEMPLATE, *args, **kwargs):
    context_class = get_context_class(name)

    if context_class:
        context = context_class()
        try:
            data = context.get_context(request, *args, **kwargs)
            if kwargs.get("context_to_string", context.context_to_string):
                data["context"] = json.dumps(data["context"])
        except UnauthorizedException as err:
            return _render(request, settings.LOGIN_CONTEXT, settings.LOGIN_TEMPLATE, data={"error" : str(err)})
    else:
        data = {}
    data.update({"debug": False})
    ret = render(request, page, data)
    return ret

def render_page(name, page=settings.ROOT_TEMPLATE, **opt):
    return lambda request, *args, name=name, page=page, opt=opt, **kwargs: _render(request, name, page=page, *args, **opt, **kwargs)

@Router.get(settings.LOGOUT_URL, need_auth=True)
def disconnect(request):
    redirect = request.GET.get("redirect", settings.LOGIN_URL)
    logout(request)
    return HttpResponseRedirect(redirect)


Router.get(settings.LOGIN_URL, need_auth=False, redirect_login=True)(render_page(settings.LOGIN_CONTEXT, settings.LOGIN_TEMPLATE))



@Router.post(settings.AUTH_URL, need_auth=False)
def auth(request):
    params = request.POST
    redirect = request.GET.get("redirect", "")
    redirect_login_error = settings.REDIRECT_LOGIN_ERROR_ARG
    user = params.get("login", None)
    password = params.get("password", None)
    if request.user and request.user.is_authenticated:
        return HttpResponseRedirect(redirect)

    if user is not None and password is not None:

        user = authenticate(request, username=user, password=password)
        if user is not None:
            login(request, user)
            return HttpResponseRedirect("/"+settings.BASE_URL)
    users = [k.username for k in User.objects.all()]
    return _render(request, settings.LOGIN_CONTEXT, settings.LOGIN_TEMPLATE, data={"error" : f"Mauvais login ou mot de passe"})

